#include <SDL.h>
#include <stdlib.h>
#include <time.h>

#define WINDOW_WIDTH 800
#define WINDOW_HEIGHT 600
#define HISTORY_MAX 10000

int main(int argc, char* argv[]) {
	// SDLの初期化
	SDL_Init(SDL_INIT_VIDEO);
	SDL_Window* window = SDL_CreateWindow(
		"random walk 2",
		SDL_WINDOWPOS_CENTERED, 
		SDL_WINDOWPOS_CENTERED,
		WINDOW_WIDTH, WINDOW_HEIGHT,
		SDL_WINDOW_SHOWN
	);
	SDL_Renderer* renderer = SDL_CreateRenderer(window, -1, 2);

	// 軌跡の座標を記録する配列
	int X[HISTORY_MAX];
	int Y[HISTORY_MAX];
	int x = WINDOW_WIDTH / 2;
	int y = WINDOW_HEIGHT / 2;
	X[0] = x; // 初期位置を記録
	Y[0] = y; // 初期位置を記録

	for (int z = 1; z <= HISTORY_MAX; z++) {
		// イベント処理
		SDL_Event event;
		int quit = 0;
		while (SDL_PollEvent(&event)) {
			if (event.type == SDL_QUIT) { quit = 1; }
		}
		if (quit) { break; }

		// 軌跡の数が最大数に達したら古いものを消す
		if (z == HISTORY_MAX) {
			X[0] = X[z - 1];
			Y[0] = Y[z - 1];
			z = 1;
		}

		// ランダムに移動方向を決定（課題：ここを書き換えてみよう）
		int dx = 0; // X軸の移動
		int dy = 0; // Y軸の移動
		int r = rand() % 4; // 0,1,2,3の乱数
		if	  (r == 0) { dx =  1; } // →右移動
		else if (r == 1) { dy =  1; } // ↓下移動
		else if (r == 2) { dx = -1; } // ←左移動
		else if (r == 3) { dy = -1; } // ↑上移動
		// 10ドット間隔で移動
		x += dx * 10;
		y += dy * 10;

		// 画面外にはみ出さないようにする処理
		if (x < 0) { x = 0; }
		if (x >= WINDOW_WIDTH) { x = WINDOW_WIDTH - 1; }
		if (y < 0) { y = 0; }
		if (y >= WINDOW_HEIGHT) { y = WINDOW_HEIGHT - 1; }

		// 軌跡の履歴を追加
		X[z] = x;
		Y[z] = y;

		// 背景を描画してクリア
		SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255);
		SDL_RenderClear(renderer);

		// 足跡の線を描画
		for (int i = 1; i <= z; i++) {
			int alpha = 64; // 透明度
			SDL_SetRenderDrawBlendMode(renderer, SDL_BLENDMODE_BLEND);
			SDL_SetRenderDrawColor(renderer, 255, 255, 255, alpha);
			SDL_RenderDrawLine(renderer, X[i-1], Y[i-1], X[i], Y[i]);
		}

		// 画面を更新
		SDL_RenderPresent(renderer);
		SDL_Delay(16); // 16ミリ秒待機 (約60fps)
	}

	// 終了
	SDL_DestroyRenderer(renderer);
	SDL_DestroyWindow(window);
	SDL_Quit();

	return 0;
}
